#!/usr/bin/env python
from gimpfu import *
import time
import datetime

# ================ *
# SERVICE ROUTINES *
# ---------------- *
#----------------------------- 
# include, then use: erro_msg 
#----------------------------- 
def erro_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(0) # box
    gimp.message ("ERROR: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
#
# ======================================================
# ANCILLARY MODULE : stretch R, G, B colours  
# ------------------------------------------------------
def StretchRGBcolours (layer, target) :

    img = layer.image
    targetB = target  
    targetW = target  

    # ============================
    # LEVELS STRETCH FOR COLOURS
    # ============================
    # RED
    
    fromV = 0.00   
    toV = 0.01   
    percent = 0
    while (percent < targetB) :  
        (mean, std_dev, median, pixels, count, percentile) = pdb.gimp_drawable_histogram (layer, HISTOGRAM_RED, fromV, toV)
        percent = percentile * 100  
        toV += 0.01    
    fromMinRED = toV - 0.01    
    
    fromV = 0.99    
    toV = 1.00    
    percent = 0
    while (percent < targetW) :  
        (mean, std_dev, median, pixels, count, percentile) = pdb.gimp_drawable_histogram (layer, HISTOGRAM_RED, fromV, toV)
        percent = percentile * 100  
        fromV -= 0.01    
    toMaxRED = fromV + 0.01   
    
    # GREEN
    
    fromV = 0.00    
    toV = 0.01   
    percent = 0 
    while (percent < targetB) :  
        (mean, std_dev, median, pixels, count, percentile) = pdb.gimp_drawable_histogram (layer, HISTOGRAM_GREEN, fromV, toV)
        percent = percentile * 100  
        toV += 0.01   
    fromMinGREEN = toV - 0.01  
    
    fromV = 0.99 
    toV = 1.00   
    percent = 0
    while (percent < targetW) :  
        (mean, std_dev, median, pixels, count, percentile) = pdb.gimp_drawable_histogram (layer, HISTOGRAM_GREEN, fromV, toV)
        percent = percentile * 100  
        fromV -= 0.01   
    toMaxGREEN = fromV + 0.01  
    
    # BLUE 

    fromV = 0.00  
    toV = 0.01    
    percent = 0
    while (percent < targetB) :  
        (mean, std_dev, median, pixels, count, percentile) = pdb.gimp_drawable_histogram (layer, HISTOGRAM_BLUE, fromV, toV)
        percent = percentile * 100  
        toV += 0.01  
    fromMinBLUE = toV - 0.01   
    
    fromV = 0.99  
    toV = 1.00   
    percent = 0
    while (percent < targetW) :  
        (mean, std_dev, median, pixels, count, percentile) = pdb.gimp_drawable_histogram (layer, HISTOGRAM_BLUE, fromV, toV)
        percent = percentile * 100  
        fromV -= 0.01  
    toMaxBLUE = fromV + 0.01   
    
    # APPLY MODIFICATIONS TO LEVELS  
    pdb.gimp_drawable_levels (layer, HISTOGRAM_RED, fromMinRED, toMaxRED, False, 1.0, 0.0, 1.0, False)    # 2.9.9
    pdb.gimp_drawable_levels (layer, HISTOGRAM_GREEN, fromMinGREEN, toMaxGREEN, False, 1.0, 0.0, 1.0, False)    # 2.9.9
    pdb.gimp_drawable_levels (layer, HISTOGRAM_BLUE, fromMinBLUE, toMaxBLUE, False, 1.0, 0.0, 1.0, False)    # 2.9.9
        
    return(layer)
#

# ----------------------------- 
# to rotate R, G, B colours
# ----------------------------- 
def RotatingColours (layer, sequence) :
    
    img = pdb.gimp_item_get_image (layer)

    # Get the layer position.
    pos = 0;
    for i in range(len(img.layers)):
        if(img.layers[i] == layer):
            pos = i
    layerName = layer.name
    if '.' in layerName:
        split = layerName.split(".", 1)
        layerName = split[0]
        
    # Create a new layer to save the results (otherwise is not possible to undo the operation).
    newLayer =layer.copy()
    img.add_layer(newLayer, pos)
    newLayer.name = layerName+" rotated"
    
    # Rotate the pixels according to the chosen sequence
    
    # Luminance (perceived option 1): (0.299*R + 0.587*G + 0.114*B)
    RtoG = 587.0/299.0*0.1+1.0
    GtoB = 114.0/587.0*0.1+1.0 
    BtoR = 299.0/114.0*0.1+1.0 
    RtoB = 114.0/299.0*0.1+1.0
    GtoR = 299.0/587.0*0.1+1.0  
    BtoG = 587.0/114.0*0.1+1.0 
    
    if sequence == 0:       #GBR
        rrgain = -1.0; rggain = RtoG; rbgain = 0.0  # R to G
        grgain = 0.0; gggain = -1.0; gbgain = GtoB  # G to B
        brgain = BtoR; bggain = 0.0; bbgain = -1.0  # B to R
    elif sequence == 1:     #BRG
        rrgain = -1.0; rggain = 0.0; rbgain = RtoB  # R to B
        grgain = GtoR; gggain = -1.0; gbgain = 0.0  # G to R
        brgain = 0.0; bggain = BtoG; bbgain = -1.0  # B to G
    elif sequence == 2:     #RBG
        rrgain = 1.0; rggain = 0.0; rbgain = 0.0  # R to R
        grgain = 0.0; gggain = -1.0; gbgain = GtoB  # G to B
        brgain = 0.0; bggain = BtoG; bbgain = -1.0  # B to G
    elif sequence == 3:     #GRB
        rrgain = -1.0; rggain = RtoG; rbgain = 0.0  # R to G
        grgain = GtoR; gggain = -1.0; gbgain = 0.0  # G to R
        brgain = 0.0; bggain = 0.0; bbgain = 1.0  # B to B
    else:                   #BGR
        rrgain = -1.0; rggain = 0.0; rbgain = RtoB  # R to B
        grgain = 0.0; gggain = 1.0; gbgain = 0.0  # G to G
        brgain = BtoR; bggain = 0.0; bbgain = -1.0  # B to R
                         
    pdb.plug_in_colors_channel_mixer (img, newLayer, False,
                                      rrgain, rggain, rbgain,
                                      grgain, gggain, gbgain,
                                      brgain, bggain, bbgain,
                                      )
    
    suffix = ["GBR","BRG","RBG","GRB","BGR"]
        
    # Change the name of the new layer (two layers can not have the same name).
    newLayer.name = layerName+" rotated "+suffix[sequence]
    
    return(newLayer)

# ================ *
#   M  A  I  N     *
# ---------------- *
   
def CrazyDrawOnDark210 (inImage, inBG, doMontage):
    
    pdb.gimp_context_push()
    
    width = inBG.width
    height = inBG.height
    WH = width+height
    source_name = inBG.name
    if '.' in source_name:
        split = source_name.split(".", 1)
        source_name = split[0]
    
    ts = time.time()
    st = datetime.datetime.fromtimestamp(ts).strftime('%Y-%m-%d %H-%M-%S')
    
    RGBImage = pdb.gimp_image_new (width, height, RGB)                    
    RGBBG = pdb.gimp_layer_new_from_drawable (inBG, RGBImage)
    RGBImage.add_layer (RGBBG, 0)
    
    Base  = StretchRGBcolours (RGBBG, 0.01)
    #pdb.gimp_display_new(NewImage)
    #return
    
    Base.mode = LAYER_MODE_NORMAL
    Base.name = "Base: Image Layer Base for Cartoon Effect"
    Base.opacity = 100.0
    
    Cartooned = RGBBG.copy()
    RGBImage.add_layer (Cartooned, 0)
    Cartooned.mode = LAYER_MODE_DIFFERENCE
    Cartooned.name = "Cartooned: Image Layer for Cartoon Effect"
    Cartooned.opacity = 100.0
    
    RADIUS=float(3.0+WH/3000.0); BLACK=float(0.100+WH/40000.000)
    pdb.plug_in_cartoon (RGBImage, Cartooned, RADIUS, BLACK)
    
    Colouring = pdb.gimp_layer_new_from_visible (RGBImage, RGBImage, "Colouring: Image Layer for more Colouring")
    RGBImage.add_layer (Colouring, 0)
    Colouring.mode = DODGE_MODE
    Colouring.opacity = 100.0
    
    Colouring2 = Colouring.copy()
    RGBImage.add_layer (Colouring2, 0)
    Colouring2.name = "Colouring2: Image Layer for more Colouring"
    Colouring3 = Colouring.copy()
    RGBImage.add_layer (Colouring3, 0)
    Colouring3.name = "Colouring3: Image Layer for more Colouring"
    
    # check
    #pdb.gimp_display_new (RGBImage)
    #return
     
    # closing
    Outcome = pdb.gimp_image_merge_visible_layers (RGBImage, CLIP_TO_BOTTOM_LAYER)
    Outcome.name = st+" "+source_name+" Outcome of DrawOnBlack"
    
    Stretched  = StretchRGBcolours (Outcome, 0.01)
    RGBBG = Stretched
    pdb.gimp_levels (RGBBG, HISTOGRAM_VALUE, 0, 255, 1.70, 0, 255)
    pdb.gimp_curves_spline (RGBBG, HISTOGRAM_VALUE, 8, (0,0,95,63,127,159,255,255))
    
    # ================================================================
    # outcomes in colours variations
    # ----------------------------------------------------------------
    
    variants = ["GBR","BRG","RBG","GRB","BGR"]

    GBRImage = RGBImage.duplicate()
    layer = GBRImage.active_layer
    name=layer.name
    rotated = RotatingColours (layer, 0)  #"GBR"
    GBRBG = pdb.gimp_image_merge_down (GBRImage, rotated, CLIP_TO_BOTTOM_LAYER)
    GBRBG.name = name+" rotated "+variants[0]
     
    BRGImage = RGBImage.duplicate()
    layer = BRGImage.active_layer
    name=layer.name
    rotated = RotatingColours (layer, 1)  #"BRG"
    BRGBG = pdb.gimp_image_merge_down (BRGImage, rotated, CLIP_TO_BOTTOM_LAYER)
    BRGBG.name = name+" rotated "+variants[1]

    RBGImage = RGBImage.duplicate()
    layer = RBGImage.active_layer
    name=layer.name
    rotated =  RotatingColours (layer, 2)  #"RBG"
    RBGBG = pdb.gimp_image_merge_down (RBGImage, rotated, CLIP_TO_BOTTOM_LAYER)
    RBGBG.name = name+" rotated "+variants[2]

    GRBImage = RGBImage.duplicate()
    layer = GRBImage.active_layer
    name=layer.name
    rotated = RotatingColours (layer, 3)  #"GRB"
    GRBBG = pdb.gimp_image_merge_down (GRBImage, rotated, CLIP_TO_BOTTOM_LAYER)
    GRBBG.name = name+" rotated "+variants[3]

    BGRImage = RGBImage.duplicate()
    layer = BGRImage.active_layer
    name=layer.name
    rotated = RotatingColours (layer, 4)  #"BGR"
    BGRBG = pdb.gimp_image_merge_down (BGRImage, rotated, CLIP_TO_BOTTOM_LAYER)
    BGRBG.name = name+" rotated "+variants[3]
     
    if doMontage == False:
        pdb.gimp_display_new (RGBImage)
        pdb.gimp_display_new (GBRImage)
        pdb.gimp_display_new (BRGImage)
        pdb.gimp_display_new (RBGImage)
        pdb.gimp_display_new (GRBImage)
        pdb.gimp_display_new (BGRImage)
  
    # -----------------------------------------------------#
    if doMontage == True:
        around = 50
        QImage = pdb.gimp_image_new (width*3+around*4, height*2+around*3, RGB)                    
        QBG = pdb.gimp_layer_new (QImage, width*3+around*4, height*2+around*3,
                                  RGB_IMAGE, "BG for Sextet Image", 100.0, LAYER_MODE_NORMAL)     
        QImage.add_layer (QBG, 0)
        gimp.set_foreground ((200,40,20))
        pdb.gimp_edit_fill (QBG, FILL_FOREGROUND)
    
        # place all siximages, the original and the 5 colours-rotated outcomes
        Q0 = pdb.gimp_layer_new (QImage, width, height, RGB_IMAGE, "To receive 1. image", 100.0, LAYER_MODE_NORMAL)      
        QImage.add_layer (Q0, 0)
        pdb.gimp_edit_copy (RGBBG)
        fs = pdb.gimp_edit_paste (Q0, True)
        pdb.gimp_floating_sel_anchor (fs)
        title = "RGB"
        gimp.set_foreground ((255,255,255,255))
        Text = pdb.gimp_layer_new (QImage, width, height, RGBA_IMAGE, "Title", 100.0, LAYER_MODE_ADDITION)
        QImage.add_layer (Text, 0)
        floats = pdb.gimp_text_fontname (QImage, Text, width*0.42, 2.0, title, 0, 0, width/15, 0, "Serif Bold")
        pdb.gimp_floating_sel_anchor (floats)
        Q0 = pdb.gimp_image_merge_down (QImage, Text, CLIP_TO_BOTTOM_LAYER)
        pdb.gimp_layer_translate (Q0, around, around)
        
        Q1 = pdb.gimp_layer_new (QImage, width, height, RGB_IMAGE, "To receive 2. image", 100.0, LAYER_MODE_NORMAL)      
        QImage.add_layer (Q1, 0)
        pdb.gimp_edit_copy (GBRBG)
        fs = pdb.gimp_edit_paste (Q1, True)
        pdb.gimp_floating_sel_anchor (fs)
        title = "GBR"
        gimp.set_foreground ((255,255,255,255))
        Text = pdb.gimp_layer_new (QImage, width, height, RGBA_IMAGE, "Title", 100.0, LAYER_MODE_ADDITION)
        QImage.add_layer (Text, 0)
        floats = pdb.gimp_text_fontname (QImage, Text, width*0.42, 2.0, title, 0, 0, width/15, 0, "Serif Bold")
        pdb.gimp_floating_sel_anchor (floats)
        Q1 = pdb.gimp_image_merge_down (QImage, Text, CLIP_TO_BOTTOM_LAYER)
        pdb.gimp_layer_translate (Q1, width+around*2, around)
         
        Q2 = pdb.gimp_layer_new (QImage, width, height, RGB_IMAGE, "To receive 3. image", 100.0, LAYER_MODE_NORMAL)      
        QImage.add_layer (Q2, 0)
        pdb.gimp_edit_copy (BRGBG)
        fs = pdb.gimp_edit_paste (Q2, True)
        pdb.gimp_floating_sel_anchor (fs)
        title = "BRG"
        gimp.set_foreground ((255,255,255,255))
        Text = pdb.gimp_layer_new (QImage, width, height, RGBA_IMAGE, "Title", 100.0, LAYER_MODE_ADDITION)
        QImage.add_layer (Text, 0)
        floats = pdb.gimp_text_fontname (QImage, Text, width*0.42, 2.0, title, 0, 0, width/15, 0, "Serif Bold")
        pdb.gimp_floating_sel_anchor (floats)
        Q2 = pdb.gimp_image_merge_down (QImage, Text, CLIP_TO_BOTTOM_LAYER)
        pdb.gimp_layer_translate (Q2, width*2+around*3, around)
        
        Q3 = pdb.gimp_layer_new (QImage, width, height, RGB_IMAGE, "To receive 4. image", 100.0, LAYER_MODE_NORMAL)      
        QImage.add_layer (Q3, 0)
        pdb.gimp_edit_copy (RBGBG)
        fs = pdb.gimp_edit_paste (Q3, True)
        pdb.gimp_floating_sel_anchor (fs)
        title = "RBG"
        gimp.set_foreground ((255,255,255,255))
        Text = pdb.gimp_layer_new (QImage, width, height, RGBA_IMAGE, "Title", 100.0, LAYER_MODE_ADDITION)
        QImage.add_layer (Text, 0)
        floats = pdb.gimp_text_fontname (QImage, Text, width*0.42, 2.0, title, 0, 0, width/15, 0, "Serif Bold")
        pdb.gimp_floating_sel_anchor (floats)
        Q3 = pdb.gimp_image_merge_down (QImage, Text, CLIP_TO_BOTTOM_LAYER)
        pdb.gimp_layer_translate (Q3, around, height+around*2)
        
        Q4 = pdb.gimp_layer_new (QImage, width, height, RGB_IMAGE, "To receive 2. image", 100.0, LAYER_MODE_NORMAL)      
        QImage.add_layer (Q4, 0)
        pdb.gimp_edit_copy (GRBBG)
        fs = pdb.gimp_edit_paste (Q4, True)
        pdb.gimp_floating_sel_anchor (fs)
        title = "GRB"
        gimp.set_foreground ((255,255,255,255))
        Text = pdb.gimp_layer_new (QImage, width, height, RGBA_IMAGE, "Title", 100.0, LAYER_MODE_ADDITION)
        QImage.add_layer (Text, 0)
        floats = pdb.gimp_text_fontname (QImage, Text, width*0.42, 2.0, title, 0, 0, width/15, 0, "Serif Bold")
        pdb.gimp_floating_sel_anchor (floats)
        Q4 = pdb.gimp_image_merge_down (QImage, Text, CLIP_TO_BOTTOM_LAYER)
        pdb.gimp_layer_translate (Q4, width+around*2, height+around*2)
        
        Q5 = pdb.gimp_layer_new (QImage, width, height, RGB_IMAGE, "To receive 3. image", 100.0, LAYER_MODE_NORMAL)      
        QImage.add_layer (Q5, 0)
        pdb.gimp_edit_copy (BGRBG)
        fs = pdb.gimp_edit_paste (Q5, True)
        pdb.gimp_floating_sel_anchor (fs)
        title = "BGR"
        gimp.set_foreground ((255,255,255,255))
        Text = pdb.gimp_layer_new (QImage, width, height, RGBA_IMAGE, "Title", 100.0, LAYER_MODE_ADDITION)
        QImage.add_layer (Text, 0)
        floats = pdb.gimp_text_fontname (QImage, Text, width*0.42, 2.0, title, 0, 0, width/15, 0, "Serif Bold")
        pdb.gimp_floating_sel_anchor (floats)
        Q5 = pdb.gimp_image_merge_down (QImage, Text, CLIP_TO_BOTTOM_LAYER)
        pdb.gimp_layer_translate (Q5, width*2+around*3, height+around*2)
         
        Sextet = pdb.gimp_image_flatten (QImage)
        Sextet.name = st+" "+source_name+" Outcome of DrawOnBlack"+" with 6 colours rotations"
        pdb.gimp_display_new (QImage)
        # ----------------------
        # delete single outcomes
        # ----------------------
        pdb.gimp_image_delete (RGBImage)
        pdb.gimp_image_delete (GBRImage)
        pdb.gimp_image_delete (BRGImage)
        pdb.gimp_image_delete (RBGImage)
        pdb.gimp_image_delete (GRBImage)
        pdb.gimp_image_delete (BGRImage)
     
    # -----------------------------------------------------------------------------------------------------
     
    pdb.gimp_context_pop()
    '''
    if doMontage == False:
        return(RGBBG)
    else:
        return(Sextet)
    '''
    return

register(
    "CrazyDrawOnDark210",
    "Gives a look of a Pencil Drawing with light contours on dark colours",
    "Gives a look of a Pencil Drawing with light contours on dark colours",
    "Diego", "Diego Nassetti ", "September 2017",
    "CrazyDrawOnDark210",
    "RGB*",
    [
      (PF_IMAGE, "image", "Input image", None),
      (PF_DRAWABLE, "idrawable", "Input drawable", None),
      (PF_TOGGLE, "montage", "Montage Outcomes?", False),
    ],
    [
      (PF_DRAWABLE, "odrawable", "Output drawable", None),  
    ],
    CrazyDrawOnDark210,
    menu="<Image>/Diego/Ancillary2",
    )

main()
